///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 *	FLEXPORTER - a flexible exporter
 *	Copyright (C) 2000-2002 Pierre Terdiman
 *	Homepage: http://www.codercorner.com/Flexporter.htm
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	This file contains the cropping values structures.
 *	\file		FLX_CropValues.h
 *	\author		Pierre Terdiman
 *	\date		February, 13, 2001
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __FLX_CROPVALUES_H__
#define __FLX_CROPVALUES_H__

	//! A possible texture matrix.
	struct FLEXPORTER_API TextureMatrix
	{
		//! Warning: that's a direct dump from MAX (untransformed)
		float	m[4][3];

		void	Identity()
		{
			ZeroMemory(m, sizeof(m));
			m[0][0] = m[1][1] = m[2][2] = 1.0f;
		}

		void	Apply(float& u, float& v, float& w)
		{
			float x = u * m[0][0] + v * m[1][0] + w * m[2][0] + m[3][0];
			float y = u * m[0][1] + v * m[1][1] + w * m[2][1] + m[3][1];
			float z = u * m[0][2] + v * m[1][2] + w * m[2][2] + m[3][2];
			u = x;
			v = y;
			w = z;
		}
	};

	//! Crop values. Useful for T-pages. Warning: equations to use them are weird.
	struct FLEXPORTER_API TextureCrop
	{
		float	OffsetU;	//!< Offset for U
		float	OffsetV;	//!< Offset for V
		float	ScaleU;		//!< Scale for U
		float	ScaleV;		//!< Scale for V

		void	Reset()
		{
			OffsetU = OffsetV = 0.0f;
			ScaleU = ScaleV = 1.0f;
		}

		// tu= (u*ScaleU) + OffsetU;
		// tv= ((v-1.0f)*ScaleV) - OffsetV;
		void	Apply(float& u, float& v)
		{
							u*=ScaleU;	u+=OffsetU;
				v-=1.0f;	v*=ScaleV;	v-=OffsetV;
		}
	};

#endif // __FLX_CROPVALUES_H__
