///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Flexine SDK main header.
 *	\file		FlexineSDK.h
 *	\author		Pierre Terdiman
 *	\date		July, 21, 2001
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __FLEXINESDK_H__
#define __FLEXINESDK_H__

	//! SDK version number
	#define	FLEXINE_SDK_VERSION		0x1000

	#define DECLARE_ICE_INTERFACE(currentclass, baseclass)									\
	public:																					\
	virtual	const char* GetInterfaceName()			const	{	return #currentclass;	}	\
	virtual	bool		IsKindOf(const char* name)	const	{								\
		if(strcmp(currentclass::GetInterfaceName(), name)==0) return true;					\
		else return baseclass::IsKindOf(name);	}

	class FLEXINE_API IceInterface
	{
		public:
		//! Constructor
		inline_							IceInterface()		{}
		//! Destructor
		virtual							~IceInterface()		{}

		virtual	const	char*			GetInterfaceName()			const	{ return "IceInterface";					}
		virtual			bool			IsKindOf(const char* name)	const	{ return strcmp("IceInterface", name)==0;	}

		virtual			bool			OnLoad()							{ return true;								}
		virtual			bool			OnClose()							{ return true;								}
	};

	class FLEXINE_API FileImporter : public IceInterface
	{
		public:
		//! Constructor
		inline_							FileImporter()		{}
		//! Destructor
		virtual							~FileImporter()		{}

						DECLARE_ICE_INTERFACE(FileImporter, IceInterface)

		virtual			const char*		GetExtension()					= 0;
	};

	//! Interface dynamic cast
	inline_ IceInterface* DynamicCast(IceInterface* it, const char* interfacename)
	{
		// Check type
		if(!it || !it->IsKindOf(interfacename))	return null;
		//
		return it;
	}

	class FLEXINE_API IcePlugin
	{
		public:
		// Constructor/Destructor
										IcePlugin()			{}
		virtual							~IcePlugin()		{}

		virtual			const char*		GetPlugName()			= 0	{ SetIceError("IcePlugin: pure virtual function called!"); return null;	}
		virtual			const char*		GetMasterAPI()			= 0	{ SetIceError("IcePlugin: pure virtual function called!"); return null;	}
		virtual			const char*		GetPlugDescription()	= 0	{ SetIceError("IcePlugin: pure virtual function called!"); return null;	}

		virtual			udword			GetNbInterfaces()		= 0	{ SetIceError("IcePlugin: pure virtual function called!"); return 0;	}
		virtual			IceInterface*	GetInterface(udword i)	= 0	{ SetIceError("IcePlugin: pure virtual function called!"); return null;	}
	};

	FUNCTION FLEXINE_API udword		GetSDKVersion();
	FUNCTION FLEXINE_API IcePlugin*	GetPlugin();

	// Exported
	typedef udword		(__stdcall* GETSDKVERSION)	();
	typedef IcePlugin*	(__stdcall* GETPLUGIN)		();

#endif // __FLEXINESDK_H__