///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains an ICE application skeleton.
 *	\file		IceApp.h
 *	\author		Pierre Terdiman
 *	\date		June, 28, 2002
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEAPP_H__
#define __ICEAPP_H__

	struct ICECORE_API ICE_EVENT
	{
		sdword	x;
		sdword	y;
		sdword	Type;
		sdword	Key;
		sdword	State;
	};

	struct ICECORE_API ICEAPPCREATE
	{
						ICEAPPCREATE();

		// WinMain parameters
		HINSTANCE		hInstance;
		HINSTANCE		hPrevInstance;
		LPSTR			lpCmdLine;
		int				nCmdShow;
		// Settings
		const char*		mAppName;		//!< Application name
		const char*		mMenuName;		//!< Possible menu name
		const char*		mScriptName;	//!< Possible script name
		sdword			mPosX;
		sdword			mPosY;
		sdword			mWidth;			//!< Width or -1 for screen size
		sdword			mHeight;		//!< Height or -1 for screen size
		BOOL			mForceSingleRun;
		BOOL			mFullscreen;
	};

	class ICECORE_API IceApp
	{
		public:
		// Constructor/Destructor
										IceApp();
		virtual							~IceApp();

		virtual				bool		Run(const ICEAPPCREATE* create);
		virtual				udword		EventProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

		protected:

		virtual				bool		Init(const ICEAPPCREATE* create);
		virtual				bool		Close();

		virtual				bool		PreInit();
		virtual				bool		PostInit();
		virtual				bool		ProcessCommandLineParam(const String& parameter);
		virtual				bool		EventLoop(const ICE_EVENT& event)	{ return true;			}
		virtual				bool		PreClose()							{ return true;			}
		virtual				bool		PostClose()							{ return true;			}

		virtual				bool		Frame(float delta_t)				{ return true;			}

		// Data access
		inline_				sdword		GetWidth()					const	{ return mWidth;		}
		inline_				sdword		GetHeight()					const	{ return mHeight;		}
		inline_				BOOL		IsFullscreen()				const	{ return mFullscreen;	}

							void		StartPause();
							void		EndPause();
		protected:
							HWND		mhWnd;
							MSG			mMsg;
							HINSTANCE	mHandle;
							int			mCmdShow;
							LPSTR		mCmdLine;
		private:
							HANDLE		mMutex;
		// Saved initialization settings
							sdword		mWidth;
							sdword		mHeight;
							BOOL		mFullscreen;
		// Time-related data
							udword		mCurrentTime;
							udword		mPrevTime;
							float		mTimeStep;
							BOOL		mPause;
		// Internal methods
							bool		CheckSingleRun(const char* app_name);
	};

	struct ICECORE_API ICEGDIAPPCREATE : ICEAPPCREATE
	{
		udword			BitPlanes;
		udword			x,y;
		const char*		Name;
	};

	class ICECORE_API GDIApplication : public IceApp
	{
		public:
		// Constructor/Destructor
										GDIApplication();
		virtual							~GDIApplication();

		override(IceApp)	bool		Init(const ICEAPPCREATE* create);
		override(IceApp)	bool		Run(const ICEAPPCREATE* create);
		override(IceApp)	bool		Close();

		inline_				ubyte*		GetLogicScreen()					{ return mLogic;				}
		inline_				void		ClearScreen()						{ ZeroMemory(mLogic, mNbBytes);	}
		inline_				void		CopyBufferToPhysic(ubyte* buffer)
										{
											IceCore::SetDIBitsToDevice(mScreenDC, 0, 0, mScreenWidth, mScreenHeight, 0, 0, 0, mScreenHeight, buffer, &mBitmapInfos, DIB_RGB_COLORS);
										}
		inline_				void		CopyLogicToPhysic()					{ CopyBufferToPhysic(mLogic);	}

							bool		InitGDI(const char* name, sdword pos_x, sdword pos_y);
							bool		OpenScreen(sdword width, sdword height, sdword bit_planes, sdword pos_x, sdword pos_y, const char* name);
							void		GetEvent(ICE_EVENT& event);
		private:
							BITMAPINFO	mBitmapInfos;
							HDC			mScreenDC;

							sdword		mScreenWidth;
							sdword		mScreenHeight;
							udword		mBitPlanes;
							udword		mNbBytes;
							ubyte*		mLogic;

							HWND		mWinID;
	};

#endif // __ICEAPP_H__
