///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains code for standard callbacks.
 *	\file		IceCallbacks.h
 *	\author		Pierre Terdiman
 *	\date		July, 8, 2001
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICECALLBACKS_H__
#define __ICECALLBACKS_H__

	struct ICECORE_API IceCallbackParams
	{
	};

	enum IceCoreCallbackCode
	{
		ICCB_REQUEST				= 0,	//!<
		ICCB_ERROR					= 1,	//!< 
		ICCB_LOG					= 2,	//!< 
		ICCB_MESSAGE				= 3,	//!< 
//		ICCB_NEWPLUGIN				= 4,	//!< 
//		ICCB_DISCARDPLUGIN			= 5,	//!< 
		ICCB_OPEN_INTERFACE			= 4,	//!< 
		ICCB_CLOSE_INTERFACE		= 5,	//!< 
		ICCB_REGISTRY_ACCESS		= 6,	//!< 
		ICCB_DUPLICATE_NAME_FOUND	= 7,	//!< 
		ICCB_EVENT_PROC				= 8,	//!<

		ICCB_SIZE					= 9,

		ICCB_FORCE_DWORD			= 0x7fffffff
	};

	typedef bool (*IceCallback)	(IceCoreCallbackCode callback_code, IceCallbackParams* params, udword user_data);

	#include "IceCallbacksDef.h"
	#include "IceCallbacksTemplateDef.h"

	FUNCTION ICECORE_API IceCoreCallbacksManager* GetCallbacksManager();

	//

	struct ICECORE_API RequestParams : public IceCallbackParams
	{
		const char*	mRequestCode;
	};

	struct ICECORE_API ErrorParams : public IceCallbackParams
	{
		const char*	mError;
		const char*	mErrrorCode;
		bool		mUpdateLog;
	};

	struct ICECORE_API MessageParams : public IceCallbackParams
	{
		const IceMsg*	mMsg;
	};

	struct ICECORE_API InterfaceParams : public IceCallbackParams
	{
		IceInterface*	mInterface;
	};

	struct ICECORE_API DuplicateParams : public IceCallbackParams
	{
		Cell*	mCurrent;		//!< Object for which conflict has been detected
		Cell*	mDuplicate;		//!< Already existing object with same name
	};

	struct ICECORE_API EventProcParams : public IceCallbackParams
	{
		HWND	mHWnd;
		UINT	mMsg;
		WPARAM	mWParam;
		LPARAM	mLParam;
	};

#endif // __ICECALLBACKS_H__
