///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains code for watchers.
 *	\file		IceWatcher.h
 *	\author		Pierre Terdiman
 *	\date		December, 26, 2001
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEWATCHER_H__
#define __ICEWATCHER_H__

	class ICECORE_API Watcher
	{
		public:
							Watcher(Cell* watched);
							~Watcher();

		//! Cast operator for const Cell* = Watcher
		inline_				operator const Cell*()	const	{ return mWatched;			}
		//! Cast operator for Cell* = Watcher
		inline_				operator Cell*()		const	{ return mWatched;			}
		//! Operator for if(Watcher)
		inline_				operator void*()				{ return (void*)mWatched;	}
		private:
				Cell*		mWatched;						//!< Watched object

		friend	class		WatcherManager;
	};

	class ICECORE_API WatcherManager
	{
		private:
		// Constructor/Destructor
							WatcherManager();
							~WatcherManager();

				bool		AddWatcher(Watcher& watcher);
				bool		RemoveWatcher(Watcher& watcher);

				bool		InvalidateWatched(const Cell* watched);

				udword		mFlags;				//!< Internal flags

				udword		mMaxNbWatchers;		//!< Max. number of watchers
				udword		mCurNbWatchers;		//!< Current number of watchers

				Watcher**	mWatchers;			//!< List of watchers
				udword*		mWatchedKIDs;		//!< List of watched KIDs
				udword*		mSortedKIDs;		//!< List of sorted watched KID indices
				udword*		mOffsets;			//!< Radix-like offsets for KID indices
				udword		mMinKID;			//!< Smallest KID index
				udword		mMaxKID;			//!< Largest KID index
		// Internal methods
				udword		GetNbWatchers(const Cell* watched);
				bool		CreateOffsets();

		friend	class		Kernel;
		friend	class		Watcher;
	};

#endif // __ICEWATCHER_H__
